/* Zadatak 3B:

Organizatori automobilisti\v{c}ke utrke ``Jezera Finske'' \v{c}uvaju podatke
o opasnim zavojima utrke u vezanoj listi. Za svaki takav zavoj pamti se
njegovo ime (string od najvi\v{s}e $20$ znakova), udaljenost od po\v{c}etka
utrke (\texttt{double}, u kilometrima), te broj de\v{z}urnih osoba na tom
zavoju (\texttt{int}).
Vezana lista zavoja je uvijek \textit{silazno\/} sortirana po udaljenosti
od po\v{c}etka utrke. Mo\v{z}ete pretpostaviti da uvijek postoji zavoj
udaljenosti nula (start utrke) i da ne postoje dva zavoja na istoj
udaljenosti od starta.
\begin{itemize}
  \item[(a)]
	Napi\v{s}ite definiciju odgovaraju\'{c}eg tipa podatka za \v{c}vor
	(element) liste zavoja, tako da je mogu\'{c}e definirati varijable
	naredbom \texttt{zavoj z}.
	Definirajte samo podatke koji su nu\v{z}ni za \v{c}uvanje takve
	liste u memoriji.
  \item[(b)]
	Napi\v{s}ite funkciju \texttt{procitaj}, koja sa standardnog ulaza
	u\v{c}itava podatke o opasnim zavojima (po jedan podatak u redu,
	tj.\ $3$ reda po stvarnom zavoju), sve dok ne u\v{c}ita
	prazno ime zavoja = oznaka za kraj niza. Funkcija treba kreirati
	silazno sortiranu listu zavoja i vratiti pokaziva\v{c} na njezin
	po\v{c}etak. Zavoji na ulazu ne moraju biti sortirani po udaljenosti
	od starta utrke.
	Dozvoljeno je kori\v{s}tenje jednog polja za u\v{c}itavanje stringa
	i funkcije \texttt{strlen}.
  \item[(c)]
	Zbog nepovoljne vremenske prognoze (obilan snijeg), organizatori
	su odlu\v{c}ili {\bf okrenuti} smjer utrke, tako da utrka starta
	iz prvobitno planiranog cilja, a zavr\v{s}ava u prvobitno planiranom
	startu.
	Napi\v{s}ite funkciju \texttt{obrni}, koja okre\'{c}e naopako
	zadanu vezanu listu zavoja prvobitne utrke. Funkcija treba vratiti
	pokaziva\v{c} na po\v{c}etak okrenute liste. Udaljenosti od novog
	starta moraju biti korektne.

	Napomena: Algoritam mora imati linearnu slo\v{z}enost u ovisnosti o
	duljini polazne liste.
\end{itemize}

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

    /* Deklaracija tipa iz (a). */

typedef struct _zavoj {
    char ime[21];
    double udaljenost;
    int broj_dezurnih;
    struct _zavoj *sljed;
} zavoj;

    /* Funkcija procitaj iz (b). */

zavoj* procitaj(void)
{
    char ime[21];    // Ime za citanje.
    int len;    // Duljina ulaznog imena.
    int i;
    char c;
    zavoj *novi, *prvi = NULL;
    zavoj *preth, *pom;    // Za sortiranje ubacivanjem na pravo mjesto.

    while (1) {    // Petlja za citanje novog zavoja, izlaz = break.

            /* Citanje imena u jednom redu (ime smije imati blankove
               pa ne koristimo %s, nego %[^\n] format).
               Pretpostavljamo da je ime dovoljno kratko da stane u string.
               U protivnom, treba dodati maksimalni broj znakova u stringu
               i petljom ucitati sve ekstra znakove do \n.
                 Unatoc tome, treba provjeriti je li ime USPJESNO ucitano,
               pa testiramo povratnu vrijednost funkcije scanf. Ako je
               jednaka 0, ime nije nadjeno -> prekidamo citanje. */
        if (scanf("%[^\n]", ime) == 0) break;

            /* Provjera duljine imena za kraj niza. */
        if ((len = strlen(ime)) == 0) break;

            /* Alokacija novog elementa liste. */
        novi = (zavoj *) malloc(sizeof(zavoj));

            /* Postavljanje nepraznog imena moze sa strcpy:
                 strcpy(novi->ime, ime);   --> priznaje se!
               ili petljom za kopiranje znak-po-znak. */
        for (i = 0; i <= len; ++i)
            novi->ime[i] = ime[i];    // Kopira i \0 znak, za i = len.

            /* Citanje ostalih podataka za novi element liste.
               Paziti na prioritet opreacija -> i &. */
        scanf("%lf", &(novi->udaljenost));
        scanf("%d", &(novi->broj_dezurnih));

            /* Za svaki slucaj, sljedbenik od novi = NULL. */
        novi->sljed = NULL;

// Kontrolni ispis ucitavanja:
        printf(" Ucitao  %s, %f, %d\n",
               novi->ime, novi->udaljenost, novi->broj_dezurnih);

            /* Sad radimo "insertion sort" u prethodno kreiranu listu.
               Za silazno sortiranje, ubaci ispred prvog manjeg od novog. */
        pom = prvi;
        preth = NULL;    // Nije nuzno.

            /* Nadji prvog manjeg - pom pokazuje na njega,
               a preth pokazuje na onog ispred. */
        while (pom != NULL && pom->udaljenost > novi->udaljenost) {
            preth = pom;
            pom = pom->sljed;
        }

            /* Ubaci novog izmedju preth i pom. */
        if (pom == prvi) {
            novi->sljed = prvi;
            prvi = novi;
        }
        else {
            novi->sljed = preth->sljed;
            preth->sljed = novi;
       }

            /* Striktno govoreci, na kraju obrade novog elementa, treba
               ucitati sve praznine iza zadnjeg broja, ukljucivo i
               prijelaz u novi red - zbog stringa ime.
               Ovo je NUZNO, inace se ucita samo prvi blok od 3 reda! */
        do {
            scanf("%c", &c);
        } while (c != '\n');

    }

        /* Vrati pokazivac na pocetak sortirane liste. */
    return prvi;
}

    /* Funkcija obrni iz (c). */

zavoj* obrni(zavoj* prvi)
{
    double max_udaljenost;    // Sluzi za okretanje udaljenosti od kraja.

    zavoj *preth, *ovaj = NULL, *sljed = prvi;
    zavoj *pom;

        /* Okretanje vezane lista - algoritam s predavanja. */
    while (sljed != NULL) {
        preth = ovaj;
        ovaj = sljed;
        sljed = sljed->sljed;
        ovaj->sljed = preth;    // Okreni sljedbenika od ovaj.
    }

        /* Ovaj = pokazivac na pocetak okrenute liste. */

        /* Ako prvi nije NULL, uzmi njegovu udaljenost = max. od
           prijasnjeg starta, i postavi sve ostale udaljenosti
           na = njegova - prijasnja (jer smo okrenuli listu). */

    if (ovaj != NULL) {
        max_udaljenost = prvi->udaljenost;
        for (pom = ovaj; pom != NULL; pom = pom->sljed)
            pom->udaljenost = max_udaljenost - pom->udaljenost;
    }

    return ovaj;
}

    /* Pomocna funkcija za ispis elemenata liste. */

void ispis(zavoj *prvi)
{
    zavoj *tmp;

    for (tmp = prvi; tmp != NULL; tmp = tmp->sljed) {
        printf(" ime = %s\n udalj = %g, dezurni = %d\n\n",
               tmp->ime, tmp->udaljenost, tmp->broj_dezurnih);
    }

    return;
}

    /* Glavni program za testiranje. */

int main(void)
{
    zavoj *prvi, *pom;

        /* Ucitavanje elemenata. */
    prvi = procitaj();

        /* Kontrolni ispis polazne (ucitane) liste. */
    printf("\nPolazna lista:\n");
    ispis(prvi);

        /* Okretanje liste. */
    prvi = obrni(prvi);

        /* Kontrolni ispis okrenute liste. */
    printf("\nOkrenuta lista:\n");
    ispis(prvi);


        /* Oslobadjanje elementa za listu. */
    while (prvi != NULL) {
        pom = prvi;
        prvi = prvi->sljed;
        free(pom);
    }

    return 0;
}
